// ECHELON.H - Neuron-C Extensions
// Copyright (c) 1990 - 2009  by Echelon Corporation
// All Rights Reserved.

#ifndef _ECHELON
#define _ECHELON

// This file contains builtin definitions for internal and external
// functions to support the Echelon (R) Neuron (R) Applications Compiler.

// NOTE! This file is AUTOMATICALLY included by the Neuron C compiler.
// You don't need to re-include it.

// Date last modified: 29-Dec-2009
//
// 29-Dec-2009:
// + added prototype for _io_spi_get_error
//
//
// 18-Dec-2008:
// + added support for I2C fast and slow mode I/O options
// + added crc16_ccitt function prototype
// 
// 18-Oct-2008:
// + added interrupt_control and _crc16_ccitt prototypes and INTERRUPT_* macros
// + added TCCLK_* macros for extended timer/counter clock rates (5000-family chips)
// + formatted this file to use spaces instead of tabs
//

// Introduce a boolean type
typedef enum boolean {
    FALSE,
    TRUE
} boolean;


// Flags used in 1st arg to 'sleep()' - can ADD or OR together.

typedef enum sleep_flags {
    COMM_IGNORE = 1,
    TIMERS_OFF = 2,
    PULLUPS_ON = 4
} sleep_flags;


// Definitions of service types for network message protocol

typedef enum service_type {
    ACKD,
    UNACKD_RPT,
    UNACKD,
    REQUEST
} service_type;


// BCD - Binary Coded Decimal - Structure used by conversion utilities.

struct bcd {
    unsigned d1:4, d2:4, d3:4, d4:4, d5:4, d6:4;
};

// Define built in macro for accessing the tick count.	Multiply
// get_tick_count()*TICK_INTERVAL to get the number of microseconds.
// Multiply this times the input clock factor (1@10Mhz, 2@5Mhz, etc)
#define get_tick_count()    (*((unsigned int *)0xffb7))
#define TICK_INTERVAL   819

// Define the external applications functions' prototypes.
// These functions are directly available to the applications programmer
// via function calls.


// Functions to perform a delay

system void delay(unsigned long count);
system void msec_delay(unsigned int milliseconds);
system void scaled_delay(unsigned long count);


// Generate an unsigned 8-bit random number

system unsigned random(void);


// Bit reversal for 8-bit quantity

system far unsigned reverse(unsigned a);


// Phase detection function

system far void init_phase_detection(boolean inverted_zero_crossing);


// Returns an NV's current length

system far unsigned get_current_nv_length(unsigned index);


// Define internal I/O functions' prototypes.
// These functions are only INDIRECTLY available to the applications
// programmer by use of the compiler's builtin functions as documented
// in the Neuron C Reference Guide.
// Use of the "hidden" pragma below allows the compiler to access these
// declarations.  However, the programmer is PROHIBITED from accessing the
// declarations as well as prohibited from redefining any of the symbols.

#pragma hidden

system unsigned _bit_input(unsigned pin);
system far unsigned _bit_input_ext(unsigned mask);
system void _bit_output_lo1(unsigned mask, unsigned data);
system void _bit_output_lo2(unsigned mask, unsigned data);
system void _bit_output_hi(unsigned mask, unsigned data);
system void _bit_output_ext(unsigned mask, unsigned data);
system far unsigned long _bitshift_input(unsigned pin, unsigned code);
system far void _bitshift_output(unsigned pin, unsigned code, unsigned long data);
system far void burst_sequence_output(unsigned tcinv, unsigned long freqOut, unsigned long* timingTable, unsigned count);
system unsigned _byte_input(void);
system void _byte_output(unsigned value);
system far unsigned long _dualslope_input(unsigned pin, unsigned mask, unsigned data);
system far void _dualslope_start(unsigned pin, unsigned long control_value);
system far unsigned _edgelog_input(unsigned long* buf_ptr, unsigned count);
system far unsigned edgelog_input_single(unsigned tcn, unsigned long* dp, unsigned max_count);
system far void edgelog_setup_single(unsigned tcn, unsigned long preload);
system void _frequency_output(unsigned pin, unsigned long control_value);

system far boolean _i2cs_read(void* read_array, unsigned address, unsigned count);
system far boolean _i2cs_write(const void* write_array, unsigned address, unsigned count);
system far boolean _i2cs_read8(void* read_array, unsigned address, unsigned count);
system far boolean _i2cs_write8(const void* write_array, unsigned address, unsigned count);
system far boolean _i2cs_read_opt8(boolean if_stop, void* read_array, unsigned address, unsigned count);
system far boolean _i2cs_write_opt8(boolean if_stop, const void* write_array, unsigned address, unsigned count);
system far boolean _i2cs_read0(void* read_array, unsigned address, unsigned count);
system far boolean _i2cs_write0(const void* write_array, unsigned address, unsigned count);
system far boolean _i2cs_read_opt0(boolean if_stop, void* read_array, unsigned address, unsigned count);
system far boolean _i2cs_write_opt0(boolean if_stop, const void* write_array, unsigned address, unsigned count);

system far boolean _i2c_read(void* read_array, unsigned address, unsigned count);
system far boolean _i2c_write(const void* write_array, unsigned address, unsigned count);
system far boolean _i2c_read8(void* read_array, unsigned address, unsigned count);
system far boolean _i2c_write8(const void* write_array, unsigned address, unsigned count);
system far boolean _i2c_read_opt8(boolean if_stop, void* read_array, unsigned address, unsigned count);
system far boolean _i2c_write_opt8(boolean if_stop, const void* write_array, unsigned address, unsigned count);
system far boolean _i2c_read0(void* read_array, unsigned address, unsigned count);
system far boolean _i2c_write0(const void* write_array, unsigned address, unsigned count);
system far boolean _i2c_read_opt0(boolean if_stop, void* read_array, unsigned address, unsigned count);
system far boolean _i2c_write_opt0(boolean if_stop, const void* write_array, unsigned address, unsigned count);

system far unsigned _ir_input(unsigned pin, unsigned* buf_ptr, unsigned max_bits, unsigned long max_pd, unsigned long threshold);
system unsigned _leveldetect_input(unsigned pin);
system far signed _magcard_input(unsigned tmo_mask, unsigned* buf_ptr);
system far signed _magt1_input(unsigned pol_mask, unsigned tmo_mask, unsigned* buf_ptr);
system far signed _magt2_input(unsigned pol_mask, unsigned tmo_mask, unsigned* buf_ptr);
system far unsigned long magx_input(unsigned pol_mask, unsigned tmo_mask, unsigned* buf_ptr, unsigned long count);
system far unsigned _muxbus_read(unsigned address);
system far unsigned _muxbus_reread(void);
system far void _muxbus_write(unsigned address, unsigned data);
system far void _muxbus_rewrite(unsigned data);
system far void _neurowire_master(unsigned dev_sel, void* buffer, unsigned count);
system far unsigned _neurowire_slave(unsigned dev_tmo_mask, void* buffer, unsigned count);
system far void _neurowire_inv_master(unsigned dev_sel, void* buffer, unsigned count);
system far unsigned _neurowire_inv_slave(unsigned dev_tmo_mask, void* buffer, unsigned count);
system unsigned _nibble_input(unsigned pin);
system void _nibble_output(unsigned pin, unsigned value);
system void _oneshot_output(unsigned pin, unsigned long control_value);
system void _parallel_input(void* micro_intf_struct_p);
system boolean _parallel_input_ready(void);
system void _parallel_output(const void* micro_intf_struct_p);
system boolean _parallel_output_ready(void);
system void _parallel_output_request(void);
system unsigned long _period_input(unsigned pin);
system void _pulsecount_output(unsigned pin, unsigned long control_value);
system void _pulsewidth_output(unsigned pin, unsigned control_value);
system long _quadrature_input(unsigned pin);
system far void _io_sci_baud(unsigned long baud_div);
system far void _io_sci_init(unsigned baud_div_hi, unsigned baud_div_lo);
system far void _io_sci_parity(unsigned parity_control);
system far void _io_sci_initram(unsigned baud_div_hi, unsigned baud_div_lo, void* isr_buffer);
system far unsigned _io_sci_get_error(void);
system far unsigned _io_sci_set_buffer_in(void* buffer, unsigned length);
system far void _io_sci_set_buffer_out(const void* buffer, unsigned length);
system far void _io_scispi_abort(void);
system far unsigned _io_scispi_input_ready(void);
system far boolean _io_scispi_output_ready(void);
system far void _io_spi_clock(unsigned clockdata);
system far void _io_spi_init(unsigned modeNM, unsigned clockdata);
system far void _io_spi_initram(unsigned modeNM, unsigned clockdata, void* isr_buffer);
system far unsigned _io_spi_get_error(void);
system far unsigned _io_spi_set_buffer(void* p, unsigned length);
system far unsigned _serial_input(void* buffer, unsigned count);
system far void _serial_output(const void* buffer, unsigned count);
system unsigned long _totalize_input(unsigned pin);
system far void _touch_read(unsigned pin, void* read_array, unsigned count);
system far void _touch_write(unsigned pin, const void* write_array, unsigned count);
system far void ext_touch_read(unsigned outpin, unsigned inpin, void* read_array, unsigned count, unsigned t_low, unsigned t_rdi, unsigned t_wrd);
system far void ext_touch_write(unsigned outpin, unsigned inpin, const void* write_array, unsigned count, unsigned t_low, unsigned t_rdi, unsigned t_wrd);
system void _triac_level_output(unsigned pin, unsigned long control_value);
system void _triac_pulse_output(unsigned pin, unsigned long control_value);
system far void _triacStrOut(unsigned tc, unsigned control_value);
system far void _triacStrInit(unsigned tc, unsigned terminal_count);
system far void _triacStrMax(unsigned tc, unsigned terminal_count);
system far unsigned _wiegand_input(unsigned tmo, unsigned pins, unsigned one_pin, unsigned* dp, unsigned nbits);


// SCI and SPI interrupt support

#pragma no_hidden

system far void io_idis(void);
system far void io_iena(void);
#pragma hidden


// Extra functions for Dallas touch I/O support

#pragma no_hidden

typedef struct search_data_s {
    int search_done;
    int last_discrepancy;
    unsigned rom_data[8];
} search_data;
#pragma hidden

system far unsigned _touch_bit(unsigned pin, unsigned data);
system far unsigned _touch_byte(unsigned pin, unsigned data);
system far int _touch_first(unsigned pin, search_data* sd);
system far int _touch_next(unsigned pin, search_data* sd);
system far int _touch_reset(unsigned pin);

system far unsigned ext_touch_bit(unsigned outpin, unsigned inpin, unsigned data, unsigned t_low, unsigned t_rdi, unsigned t_wrd);
system far unsigned ext_touch_byte(unsigned outpin, unsigned inpin, unsigned data, unsigned t_low, unsigned t_rdi, unsigned t_wrd);
system far int ext_touch_first(unsigned outpin, unsigned inpin, search_data* sd, unsigned t_low, unsigned t_rdi, unsigned t_wrd);
system far int ext_touch_next(unsigned outpin, unsigned inpin, search_data* sd, unsigned t_low, unsigned t_rdi, unsigned t_wrd);
system far int ext_touch_reset(unsigned outpin, unsigned inpin);


// The I/O select functions - internal definitions

system void _select_input_fn(unsigned pin_mode_clock, unsigned mode);
#pragma no_hidden

typedef enum io_direction {
    IO_DIR_IN,
    IO_DIR_OUT
} io_direction;
#pragma hidden

system void _io_direction_lo(unsigned mask, io_direction dir);
system void _io_direction_hi(unsigned mask, io_direction dir);

// The I/O set clock functions - internal definitions

system void _io_set_clock(unsigned pin_and_clock);
system far void _io_set_clock_x2(unsigned pin_and_clock);

// The I/O initialization functions - internal definitions

system void _init_baud(unsigned rate_select);
system void _init_timer_counter1(unsigned tcn_ck, unsigned mode);
system void _init_timer_counter2(unsigned tcn_ck, unsigned mode);

// Functions dealing with aspects of io_changes - internal definitions

system void _io_change_init(unsigned index, unsigned long io_value);
system long _io_input_value(void);

// The I/O preserve-input function - internal definitions

system far void _io_abort_clear(unsigned tcn);


// Miscellaneous ROM functions called by the compiler - "helper functions"

// To poll network variables and propagate network variables

system void _nv_array_poll(unsigned end_offset, unsigned start_index);
system void _nv_poll(unsigned index);
system void _nv_poll_all(void);
system far void _nv_array_update_request(unsigned end_offset, unsigned start_index);
system void _nv_update_request(unsigned index);
system far void _nv_update_request_all(void);

// Supporting messaging
system boolean _msg_alloc(void);
system boolean _msg_alloc_priority(void);
system void _msg_cancel(void);
system void _msg_free(void);
system boolean _msg_receive(void);
system void _msg_send(void);

system unsigned _msg_auth_get(void);
system unsigned _msg_code_get(void);
system far void _msg_data_blockget(unsigned len, void* p, unsigned offset);
system unsigned _msg_data_get(unsigned offset);
system far boolean _msg_duplicate_get(void);
system unsigned _msg_len_get(void);
system far unsigned _msg_rcvtx_get(void);
system service_type _msg_service_get(void);

system void _msg_auth_set(unsigned authenticate);
system void _msg_code_set(unsigned code);
system void _msg_data_blockset(unsigned len, const void* p, unsigned offset);
system void _msg_data_set(unsigned offset, unsigned value);
system void _msg_priority_set(unsigned priority_on);
system void _msg_service_set(service_type service);
system void _msg_tag_set(unsigned tag);

system boolean _resp_alloc(void);
system void _resp_cancel(void);
system void _resp_free(void);
system boolean _resp_receive(void);
system void _resp_send(void);

system void _resp_code_set(unsigned code);
system far void _resp_data_blockset(unsigned len, const void* p, unsigned offset);
system void _resp_data_set(unsigned offset, unsigned value);


// Support functions for event expressions

system boolean _flush_completes(void);
system boolean _io_changes(unsigned index, unsigned long io_value);
system boolean _io_changes_by(unsigned index, unsigned long io_value, unsigned long min_delta);
system boolean _io_changes_to(unsigned index, unsigned long io_value, unsigned long target_value);
system boolean _io_update_occurs(unsigned long io_value);
system boolean _msg_arrives(void);
system boolean _msg_code_arrives(unsigned code);
system boolean _msg_completes(unsigned tag);
system boolean _msg_fails(unsigned tag);
system boolean _msg_succeeds(unsigned tag);
system boolean _offline(void);
system boolean _online(void);
system boolean _nv_array_update_completes(unsigned end_offset, unsigned start_index);
system boolean _nv_array_update_fails(unsigned end_offset, unsigned start_index);
system boolean _nv_array_update_occurs(unsigned end_offset, unsigned start_index);
system boolean _nv_array_update_succeeds(unsigned end_offset, unsigned start_index);
system boolean _nv_update_completes(unsigned index);
system boolean _nv_update_fails(unsigned index);
system boolean _nv_update_occurs(unsigned index);
system boolean _nv_update_succeeds(unsigned index);
system boolean _resp_arrives(unsigned tag);
system boolean _timer_expires(unsigned n);
system boolean _timer_expires_any(void);
system boolean _wink(void);


// Library functions which support "is_bound".
system boolean _bound_mt(unsigned tag);
system far boolean _bound_nv(unsigned nv_index);


// Miscellaneous library functions - Neuron C programmer uses
// documented builtin prototypes for "memset, memcpy, bin2bcd, bcd2bin".
// These are identical except the Neuron C names don't have underscores.

system far void _bin2bcd(unsigned long val, struct bcd* p);
system far unsigned long _bcd2bin(const struct bcd* p);
system far void _fblock_director(unsigned index, int cmd);
system void _memcpy(void* dest, const void* src, unsigned len);
system void _memcpy8(void* dest, const void* src, unsigned len);
system far void _memcpy16(void* dest, const void* src, unsigned long len);
system void _memset(void* ptr, unsigned val, unsigned len);
system void _memset8(void* ptr, unsigned val, unsigned len);
system far void _memset16(void* ptr, unsigned val, unsigned long len);
system void _sleep(unsigned flags_and_wakeup_device);


// Misc global system variables - the compiler modifies the names below by
// removing the trailing underscore.  This is done to get the type information
// for these symbols into the debug file.  The actual symbols can't be used
// directly because the actual symbols are all keywords in Neuron C.

system boolean input_is_new_;
system signed long input_value_;
system unsigned short msg_tag_index_;
system unsigned short nv_array_index_;
system unsigned short nv_in_index_;

#pragma no_hidden


// I/O function to preload the TCs for the edgelog device

system far void io_edgelog_preload(unsigned long value);

// Touch (1-wire) I/O support for "strong pullup" functionality
extern system far int touch_reset_spu(unsigned pinmask);
extern system far void touch_byte_spu(unsigned pinmask, unsigned data);
extern system far void touch_read_spu(unsigned pinmask, unsigned *dp, unsigned count);
extern system far void touch_write_spu(unsigned pinmask, const unsigned *dp, unsigned count);

// Network variable source address support

typedef struct nv_in_addr_t {
    unsigned    domain  : 1;
    unsigned    flex_domain : 1;
    unsigned    format  : 6;

    // Interpret 'format' as:	0 => Broadcast,
    //	1 => Group,		2 => Subnet/Node,
    //	3 => Neuron-ID,		4 => Turnaround
    //	See Addrdefs.h for enumerated type ADDR_FORMAT
    struct {
        unsigned subnet;
        unsigned    : 1;
        unsigned node   : 7;
    } src_addr;

    struct {
        unsigned group;
    } dest_addr;
} nv_in_addr_t;

system const nv_in_addr_t nv_in_addr;


// Support for LonMark features

// CP value and template files

extern unsigned short   cp_template_file[];
extern unsigned short   cp_modifiable_value_file[];
extern unsigned short   cp_readonly_value_file[];

// The following definitions are for the lengths of the template file and
// the value files.  They are NOT arrays, no matter how they are declared!
// Do NOT use these "fake" values except via the macros below ...

#define cp_template_file_len ((unsigned long)(cp_template_file_len_fake))
#define cp_modifiable_value_file_len ((unsigned long)(cp_modifiable_value_file_len_fake))
#define cp_readonly_value_file_len ((unsigned long)(cp_readonly_value_file_len_fake))

extern const unsigned short  cp_template_file_len_fake[];
extern const unsigned short  cp_modifiable_value_file_len_fake[];
extern const unsigned short  cp_readonly_value_file_len_fake[];

// Fblock map for NVs

extern const unsigned short  fblock_index_map[];

// The following definitions are used for the interrupt system. The application
// will not build correctly unless the target chip supports all features required
// by your application.
// Use a combination of the INTERRUPT_* macros for the argument of the interrupt_control
// function. INTERRUPT_IO enables the I/O interrupts, INTERRUPT_TC the timer/counter
// interrupt, and INTERRUPT_REPEATING enables the repeating system timer interrupt.
// You can only enable interrupts that are supported by your application through the
// declaration of corresponding interrupt tasks.
//
#define INTERRUPT_IO    0x03
#define INTERRUPT_TC    0x0C
#define INTERRUPT_REPEATING 0x10

extern void interrupt_control(unsigned irqSelect);

// Hardware-supported CRC-16, available with 5000-family chips:
extern system far unsigned long crc16_ccitt(unsigned long crc_in, const unsigned* sp, unsigned len);

// Interrupt-safe SCI input setup for 5000-family chips:
extern system far unsigned sci_in_request_ex(void* buffer, unsigned length);

/*
 * Extended Timer/Counter Clock Rates
 *
 * The following definitions are provided to specify the timer/counter 
 * input clock (see io_set_clock() function). Note only values 0-7 
 * (TCCLK_5MHz..TCCLK_39k06Hz) are available with 3100-family parts; 
 * the full range is supported with 5000-family parts.
 */
#define TCCLK_40MHz   13        // 40MHz, 5000-family parts only 
#define TCCLK_20MHz   14        // 20MHz, 5000-family parts only  
#define TCCLK_10MHz   15        // 10MHz, 5000-family parts only  
#define TCCLK_5MHz     0        // 5MHz 
#define TCCLK_2500kHz  1        // 2.5MHz
#define TCCLK_1250kHz  2        // 1.25MHz
#define TCCLK_625kHz   3        // 625kHz
#define TCCLK_312k5Hz  4        // 312.5kHz
#define TCCLK_156k2Hz  5        // 156.2kHz
#define TCCLK_78k12Hz  6        // 78.12kHz
#define TCCLK_39k06Hz  7        // 39.06kHz
#define TCCLK_19k53Hz  8        // 19.53kHz, 5000-family parts only 
#define TCCLK_9k77Hz   9        // 9.77kHz, 5000-family parts only 
#define TCCLK_4k88Hz  10        // 4.88kHz, 5000-family parts only 
#define TCCLK_2k44Hz  11        // 2.44kHz, 5000-family parts only 
#define TCCLK_1k22Hz  12        // 1.22kHz, 5000-family parts only 

// Do not include other files by placing #include directives in this file.
// Otherwise, symbols from this file may not appear in the DBG information
// file properly.

// end of: ECHELON.H
#endif
